Package: exploreSE
Title: Explorer for Your Bulk RNA Seq Analyses
Version: 0.99.5
Authors@R: 
    person("Jasper", "Spitzer", , "jasspitzer135@gmail.com", 
        role = c("aut", "cre"), comment = c(ORCID="0000-0001-9696-2092"))
Description: Interactive exploration of a Summarized experiment and associated DE analyses. A shiny
 app to inspect and interact with pre-calculated differential expression and functional 
 enrichment results, and to facilitate comparisons between multiple models. This is meant
 to be a tool for exploration and discussion, rather than analysis.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
Depends: R (>= 4.6.0)
Imports: BiocGenerics, clusterProfiler, colourpicker, DeeDeeExperiment,
        DESeq2, dplyr, DT, forcats, ggplot2, htmltools, magrittr,
        matrixStats, methods, msigdbr, org.Hs.eg.db, org.Mm.eg.db,
        plotly, purrr, readr, rlang, S4Vectors, shiny, shinyWidgets,
        stats, stringr, SummarizedExperiment, tibble, tidyr, tidyselect
Suggests: airway, BiocStyle, knitr, rmarkdown, testthat (>= 3.0.0)
URL: https://github.com/jaspitzer/exploreSE
BugReports: https://github.com/jaspitzer/exploreSE/issues
biocViews: ShinyApps, RNASeq, Visualization, GeneExpression, Pathways,
        GO
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/exploreSE
git_branch: devel
git_last_commit: c0866f8
git_last_commit_date: 2026-08-18
Repository: Bioconductor 3.24
Date/Publication: 2026-08-20
NeedsCompilation: no
Packaged: 2026-08-20 21:57:59 UTC; biocbuild
Author: Jasper Spitzer [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9696-2092>)
Maintainer: Jasper Spitzer <jasspitzer135@gmail.com>
