utils::globalVariables(c(
  "padj",
  "PC1",
  "PC2",
  "group",
  "s_a_m_p_l_e",
  "g_r_o_u_p",
  "log2FoldChange",
  ".",
  "gene_id",
  "gene_name",
  "sig",
  "highlighted",
  "display_category",
  "p.adjust",
  "FoldEnrichment",
  "Description",
  "geneID",
  "NES",
  "BR",
  "BgRatio",
  "GR",
  "GeneRatio",
  "dds",
  "condition",
  "gs_name",
  "gene_symbol",
  "gene",
  "name",
  "value",
  "sd",
  "mean_expr",
  "sd_group1",
  "sd_group2",
  "ranking",
  "mean_expr_group2",
  "mean_expr_group1",
  "n",
  "direction",
  "ensembl_gene",
  "ncbi_gene",
  "sample_ident",
  "gene_ident",
  "sig_A",
  "sig_B",
  "log2FoldChange_A",
  "log2FoldChange_B"
))
